@@ -3,21 +3,18 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
+ * The contents of this file constitute Original Code as defined in and
+ * are subject to the Apple Public Source License Version 1.2 (the
+ * "License").  You may not use this file except in compliance with the
+ * License.  Please obtain a copy of the License at
+ * http://www.apple.com/publicsource and read it before using this file.
  * 
- * This file contains Original Code and/or Modifications of Original Code
- * as defined in and that are subject to the Apple Public Source License
- * Version 2.0 (the 'License'). You may not use this file except in
- * compliance with the License. Please obtain a copy of the License at
- * http://www.opensource.apple.com/apsl/ and read it before using this
- * file.
- * 
- * The Original Code and all software distributed under the License are
- * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * This Original Code and all software distributed under the License are
+ * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
- * Please see the License for the specific language governing rights and
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.  
+ * Please see the License for the specific language governing rights and 
  * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
@@ -155,10 +152,8 @@ enum {
 
 
 enum {
-    kHIDNoInterfaceProtocol		= 0,
-    kHIDKeyboardInterfaceProtocol 	= 1,
-    kHIDMouseInterfaceProtocol		= 2,
-    kUSBVendorSpecificProtocol		= 0xff
+    kHIDKeyboardInterfaceProtocol = 1,
+    kHIDMouseInterfaceProtocol	= 2
 };
 
 
@@ -197,14 +192,7 @@ enum {
     kUSBMassStorageUFISubClass 		= 0x04,
     kUSBMassStorageSFF8070iSubClass 	= 0x05,
     kUSBMassStorageSCSISubClass 	= 0x06,
-    kUSBHIDBootInterfaceSubClass 	= 0x01,
-    kUSBCommDirectLineSubClass	= 0x01,
-    kUSBCommAbstractSubClass	= 0x02,
-    kUSBCommTelephoneSubClass	= 0x03,
-    kUSBCommMultiChannelSubClass	= 0x04,
-    kUSBCommCAPISubClass		= 0x05,
-    kUSBCommEthernetNetworkingSubClass	= 0x06,
-    kUSBATMNetworkingSubClass	= 0x07
+    kUSBHIDBootInterfaceSubClass 	= 0x01
 };
 
 /* DFU Class Attributes */
